@@ -1,6 +1,6 @@
 package App::Greple::xlate::deepl;
 
-our $VERSION = "0.3202";
+our $VERSION = "0.33";
 
 use v5.14;
 use warnings;
@@ -66,7 +66,7 @@ sub xlate_each {
 }
 
 sub xlate {
-    my @from = map { /\n\z/ ? $_ : "$_\n" } @_;
+    my @from = @_;
     my @to;
     my $max = $App::Greple::xlate::max_length || $param{$method}->{max} // die;
     while (@from) {
